/*	ATTINY104_LED_DRIVER
*	Created: 07/07/2016 08:20:00 PM
*	Author : Ripper121
*	Drive up to 10 WS2812b Led's over Uart
*	Protocol: 1 Start Byte (0x01), (Led count * 3) Color Bytes, 1 CRC Byte
*	ACK = 0xFF, NAK = 0x00
*	FOSC=1000000 :"After powering up the device or after a reset the system clock is automatically
*	set to calibrated internal 8MHz oscillator, divided by 8" (Datasheet pag.32)
*   But we need at least 8MHz so we set the divider to 1
				  __________
			VCC--|1       14|--GND (TPI CLK)
			PA0--|2       13|--PB3 (CDC TX) -->USART_Transmit()
(TPI DATA)	PA1--|3       12|--PB2 (CDC RX) <--USART_Receive()
(RESET)		PA2--|4       11|--PB1 (BUTTON)
			PA3--|5       10|--PB0
			PA4--|6        9|--PA7
(LED)		PA5--|7        8|--PA6
				 \__________/
Atmel ATtiny104/102 Xplained Nano
*/
#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/interrupt.h>
#include "ws2812.h"
#define LED_PIN    (1 << PA5)
#define BUTTON_PIN (1 << PB1)

unsigned char packCRC(unsigned char *s, unsigned char length)
{
	unsigned char c = 0;
	for (unsigned char i = 0; i < length; i++) {
		c ^= s[i];
	}
	return c;
}

void USART_Transmit( unsigned char data )
{
	/* Wait for empty transmit buffer */
	while ( !( UCSRA & (1<<UDRE)) );
	/* Put data into buffer, sends the data */
	UDR = data;
}

unsigned char USART_Receive( void )
{
	/* Wait for data to be received */
	while ( !(UCSRA & (1<<RXC)) );
	/* Get and return received data from buffer */
	return UDR;
}

void USART_Flush( void )
{
	unsigned char dummy;
	while ( UCSRA & (1<<RXC) ) dummy = UDR;
}

/*----------------------------------------------------------------------------------------------------------------------------------------------------------*/
#define LEDPIX 6 //<- Max is 10
#define MAXPIX LEDPIX*3

int main(void){
	CCP = 0xD8; //Set Unsecure
	CLKPSR = 0; //Clockdiv 0 -> 8Mhz
	PUEB |= BUTTON_PIN;         // Enable Pull-Up function in PB1.
	PORTB |= BUTTON_PIN;        // Set Pull-Up for the Button.
	DDRA |= LED_PIN;            // Configure LED pin as Output.
	/*Set baud rate */
	UBRRH = (unsigned char)(51>>8);//9600 Baud @ 8Mhz
	UBRRL = (unsigned char)51;//9600 Baud @ 8Mhz
	/*Enable receiver and transmitter */
	UCSRB = (1<<RXEN)|(1<<TXEN);
	/* Set frame format: 8data, 2stop bit */
	UCSRC = (1<<USBS)|(3<<UCSZ0);
	_delay_ms(250);
	PORTA |= LED_PIN;         // Switch off the LED.
	
	while(1){
		unsigned char rxLedPack[MAXPIX],crcRGB = 0x00,rxcrcRGB = 0x00;
		if(UCSRA & (1<<RXC)){
			if(USART_Receive()==0x01){
				for(unsigned char i=0;i<MAXPIX;i++){
					rxLedPack[i]=USART_Receive();
				}
				crcRGB = packCRC(rxLedPack,MAXPIX);
				rxcrcRGB = USART_Receive();
				USART_Flush();
				if(rxcrcRGB==crcRGB){
					ws2812_sendarray((unsigned char *)rxLedPack,MAXPIX);
					USART_Transmit(0xFF);
				}
				else
				{
					for(unsigned char i=0;i<MAXPIX;i++){
						rxLedPack[i]=0x00;
					}
					ws2812_sendarray((unsigned char *)rxLedPack,MAXPIX);
					USART_Transmit(0x00);
				}
			}
		}
	}
}